#!/usr/bin/env python

"""
usage: pyre_client05.py [-h] [-v] [-d DELAY]

synopsis:
  A simple Pyre client.  Sends response to dealer.

optional arguments:
  -h, --help            show this help message and exit
  -v, --verbose         print messages during actions.
  -d DELAY, --delay DELAY
                        delay/sleep and pretend to work for x seconds

examples:
  python pyre_client05.py
"""

from __future__ import print_function
#from six.moves import input
#import sys
import argparse
import socket
import uuid
import json
import time
import pyre


Node_name = socket.gethostname()
Node_value = '{} server'.format(Node_name)
Group_name = 'group1'


def dbg_print(opts, msg, **kwargs):
    """Print a message if verbose is on."""
    if opts.verbose:
        print(msg, **kwargs)


def run(opts):
    node = pyre.Pyre(Node_name)
    node.set_header(Node_name, Node_value)
    node.start()
    node.join(Group_name)
    idx = 0
    while True:
        try:
            print('ready to receive')
            while True:
                data = node.recv()
                if data[0] == b'WHISPER':
                    break
            dbg_print(opts, '{}. received data: {}'.format(idx, data))
            # Send a response back to the sender.
            # Get the sender's uuid.
            time.sleep(opts.delay)
            uuid1 = data[1]
            uuid2 = uuid.UUID(bytes=uuid1)
            data1 = data[3]
            data1 = data1.decode()
            data2 = json.loads(data1)
            data3 = data2['request']
            #
            # Perform a task and create a response to the "request".
            data4 = data3.upper()
            data2['response'] = data4
            data2['sender'] = Node_name
            data5 = json.dumps(data2)
            node.whispers(uuid2, data5)
            print('{}. sending response: {}\n----------'.format(
                idx, data4))
            idx += 1
        except KeyboardInterrupt:
            break
    print('\ncleaning up and leaving')
    node.leave(Group_name)
    node.stop()


def main():
    description = """\
synopsis:
  A simple Pyre client.  Sends response to dealer.
"""
    epilog = """\
examples:
  python pyre_client05.py
"""
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="print messages during actions.",
    )
    parser.add_argument(
        "-d", "--delay",
        type=float, default=0.0,
        help="delay/sleep and pretend to work for x seconds",
    )
#    parser.add_argument(
#        "command",
#        help="A command, one of: getlist, getone, add, update, delete"
#    )
#    parser.add_argument(
#        "-a", "--args",
#        help="Arguments: (1) task name or (2) form parameters"
#    )
    options = parser.parse_args()
    run(options)


if __name__ == '__main__':
    # import pdb; pdb.set_trace()
    # import ipdb; ipdb.set_trace()
    main()
